Package: DenoIST
Title: DenoIST: Denoising Image-based Spatial Transcriptomics data
Version: 1.1.0
Authors@R: c( 
    person("Aaron", "Kwok", , "akwok@svi.edu.au", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-7831-4198")),
    person("Heejung","Shim", role=c("aut")),
    person("Davis","McCarthy", role=c("aut")))
biocViews: Software, Preprocessing, Spatial, GeneExpression,
        SingleCell, Transcriptomics
Description: DenoIST identifies and removes contamination in Image-based Spatial Transcriptomics data, using a transposed poisson mixture model with local neighbourhood offsets to infer genes that are likely to be due to neighbourhood contamination rather than endogenous expression.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
URL: https://github.com/aaronkwc/DenoIST
BugReports: https://github.com/aaronkwc/DenoIST/issues
Imports: flexmix, hexbin, pbapply, sparseMatrixStats,
        SpatialExperiment, stats, SummarizedExperiment, parallel,
        Matrix, dbscan, methods
Suggests: BiocStyle, knitr, rmarkdown, testthat, ggplot2, patchwork
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/DenoIST
git_branch: devel
git_last_commit: 35bfd0d
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:21:12 UTC; biocbuild
Author: Aaron Kwok [aut, cre] (ORCID: <https://orcid.org/0000-0001-7831-4198>),
  Heejung Shim [aut],
  Davis McCarthy [aut]
Maintainer: Aaron Kwok <akwok@svi.edu.au>
Depends: R (>= 3.5.0)

# Generated by roxygen2: do not edit by hand

export(denoist)
export(local_offset_distance_with_background)
export(local_offset_distance_with_background_fast)
export(solve_poisson_mixture)
importFrom(Matrix,sparseMatrix)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,assay)
importFrom(dbscan,frNN)
importFrom(flexmix,FLXMRglm)
importFrom(flexmix,clusters)
importFrom(flexmix,flexmix)
importFrom(flexmix,parameters)
importFrom(hexbin,hexbin)
importFrom(methods,as)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(pbapply,pblapply)
importFrom(sparseMatrixStats,rowSums2)
importFrom(stats,dpois)
importFrom(stats,runif)
importFrom(stats,xtabs)

# DenoIST: Denoising Image-based Spatial Transcriptomics data

<!-- badges: start -->

[![R-CMD-check](https://github.com/aaronkwc/DenoIST/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/aaronkwc/DenoIST/actions/workflows/R-CMD-check.yaml)

<!-- badges: end -->

## Overview

DenoIST is a package for denoising image-based spatial transcriptomics data. It takes a IST count matrix and returns a adjusted count matrix with contamination removed.

The package is designed to be used with the [SpatialExperiment](https://bioconductor.org/packages/release/bioc/html/SpatialExperiment.html) class. If you are using a different format, a matrix input with a data frame of coordinates can also be accepted.

This is still very much a work in progress and we are still working on the documentation. Please feel free to open an issue or email at akwok@svi.edu.au if you have any questions or suggestions.

## News:
- **2026-03-17**: Major update
  - Added `local_offset_distance_with_background_fast` function which uses `dbscan` for faster neighbour finding. Also fixed critical bug that causes wrong distance calculation.
  - `denoist()` now defaults to fast neighbour finding via the `neighbour_mode` option.

- **2026-02-24**: Minor update
  - Fixed minor bug where background offset cannot be calculated because an entire gene gets filtered out because of low qv. This should not change existing usage as the issue only arises in extremely small toy datasets.
  - `n_inits` can now be tuned in the `denoist()` function for speed.

- **2025-06-25**: Major update
  - Fixed memory usage for parallel processing. Feature is available only on linux/UNIX machines due to dependency on `parallel`.
  - Posterior cutoff is now a tunable parameter.
  - QOL changes including checking input types and error handling.

- **2025-05-20**: Initial release of DenoIST. The package is now available on and GitHub.

## Installation:

From Bioconductor:

```{r install}
if(!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("DenoIST")
```

Or from Github directly:

```         
BiocManager::install(c('sparseMatrixStats', 'SpatialExperiment','SummarizedExperiment'))
devtools::install_github("aaronkwc/DenoIST")
```

## Quick start:

In most cases, you will only need to use the `denoist()` wrapper function.

It takes 2-3 inputs:

1.  `mat` : SpatialExperiment object (with the counts in assay() slot) or a count matrix with genes as rows and cells as columns.
2.  `tx`: Transcript data frame (a data frame with each row being an individual transcript, with columns specifying each transcripts' coordinates and qv). If your transcript file is not from Xenium and has no qv score, you can set a dummy column of `qv = 20` for all transcripts. This workaround should not be needed in future updates.
3.  `coords`: If using a count matrix, a data frame (cells x 2) for each cell's centroid 2D coordinate.

The function will return a list with

1.  `adjusted_counts`: The adjusted counts matrix with contamination removed.
2.  `memberships`: A data frame with the inferred identity of each gene in each cell (1 for real or 0 for contamination).
3.  `params`: A list with the estimated parameters used in the model. The posterior probabilities of each gene being real or contamination can be found in `params$posterior_probs`, higher means more likely to be contamination.

You can run `?denoist` for more details on the extra parameters you can adjust.

## Examples

With a SpatialExperiment object:

```         
library(DenoIST)
library(SpatialExperiment)

res <- denoist(mat = spe,
              tx = tx,
              coords = NULL,
              distance = 50, nbins = 200, cl = 1,
              out_dir = "denoist_results")
```

With a count matrix and coordinates:

```         
library(DenoIST)

res <- denoist(mat = mat,
               tx = tx,
               coords = coords,
               distance = 50, nbins = 200, cl = 1,
               out_dir = "denoist_results")
```

## Vignette

Check out the vignette to get started:

```
library(DenoIST)
browseVignettes("DenoIST")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
suppressPackageStartupMessages({
  library(DenoIST)
  library(SpatialExperiment)
  library(ggplot2)
  library(patchwork)
})

## ----load_spe-----------------------------------------------------------------
spe <- readRDS(system.file("extdata", "example_spe.rds", package = "DenoIST"))
spe

## ----load_data----------------------------------------------------------------
tx <- readRDS(system.file("extdata", "tx_sub.rds", package = "DenoIST"))
tx

## ----denoist------------------------------------------------------------------
# DenoIST
result <- denoist(mat = spe, tx = tx, coords = NULL, tx_x = "x_location", tx_y = "y_location", feature_label = "feature_name", distance = 50, nbins = 200, cl = 1)

## ----read_results-------------------------------------------------------------
result <- readRDS(system.file("extdata", "result.rds", package = "DenoIST"))

## ----check_results------------------------------------------------------------
result$adjusted_counts[1:5, 1:5]

## -----------------------------------------------------------------------------
result$memberships[1:5, 1:5]

## -----------------------------------------------------------------------------
result$params[[42]]

## -----------------------------------------------------------------------------
# a custom helper function for plotting
plot_feature_scatter <- function(coords, mat, feature, size = 0.3, output_filename = NULL) {
  # Create a data frame from the coordinates and features
  plot_data <- data.frame(x = coords[, 1], y = coords[, 2], feature = mat[feature,])
  
  # Create the scatterplot
  p <- ggplot(plot_data, aes(x = x, y = y, color = feature)) +
    geom_point(size = size, alpha = 0.5) +  # Make the dots smaller
    theme_minimal() +
    labs(title = feature, x = "X Coordinate", y = "Y Coordinate", color = "Feature") +
    theme(legend.position = "right") +
    scale_color_viridis_c()  # Use a color palette with higher contrast
  
  # Save the plot if an output filename is provided
  if (!is.null(output_filename)) {
    ggsave(output_filename, p, width = 10, height = 8, units = "in", dpi = 300)
  }
  
  # Return the plot
  return(p)
}

## ----fig.height=8, fig.width=10-----------------------------------------------
orig <- plot_feature_scatter(coords = spatialCoords(spe), log(assay(spe)+1), feature = "EPAS1") + ggtitle('Original')
adj <- plot_feature_scatter(coords = spatialCoords(spe), log(result$adjusted_counts+1), feature = "EPAS1") + ggtitle('DenoIST adjusted')
(orig + adj) + plot_layout(guides = "collect")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' This code is used for generating the small test dataset for vignette

library(SpatialExperimentIO)

dir = "/mnt/beegfs/mccarthy/backed_up/general/rlyu/Dataset/LFST_2022/GEO_2025/VUHD116A/relabel_output-XETG00048__0003817__VUHD116A__20230308__003730/outs/"
spe <- readXeniumSXE(dir, returnType = "SPE")
example_small <- spe[,sample(ncol(spe), 300)]
saveRDS(example_small, "inst/extdata/example_spe.rds")

#' @title Neighbourhood offset
#' @description This function calculates the local neighbourhood offset together with ambient background for each cell in a count matrix.
#' @param mat A count matrix with genes as rows and cells as columns.
#' @param tx A transcript dataframe with x, y coordinates and qv values.
#' @param coords A dataframe with x, y coordinates of each cell as separate columns.
#' @param tx_x Column name for the x coordinates in the transcripts dataframe.
#' @param tx_y Column name for the y coordinates in the transcripts dataframe.
#' @param feature_label Column name for the gene of each transcript in the transcripts dataframe.
#' @param distance The maximum distance to consider for local background estimation.
#' @param nbins The number of bins to use for hexagonal binning, used for calculating background transcript contamination.
#' @param cl The number of cores to use for parallel processing.
#' @param verbose Logical, if TRUE, print progress messages.
#' @return A matrix of local background counts for each gene in each cell.
#' @details
#' The function calculates the offset used for each cell based on their local
#' neighbourhoods.In most cases you do not need to use this as denoist already
#' runs this internally but it is good for debugging if needed.
#' @examples
#' # Load example data
#' set.seed(42)
#' mat <- matrix(rpois(1000, lambda = 10), nrow = 10, ncol = 100)
#' rownames(mat) <- paste0("gene", 1:10)
#' coords <- data.frame(x = rnorm(100), y = rnorm(100))
#' tx <- data.frame(x = c(rnorm(500), rnorm(500, 3)),
#'                  y = c(rnorm(500), rnorm(500, 3)),
#'                  qv = rep(30, 1000), gene = paste0('gene', 1:10))
#' # Run DenoIST
#' off_mat <- local_offset_distance_with_background(mat, tx, coords,
#'                                                  distance = 1, nbins = 50,
#'                                                  cl = 1, verbose = TRUE)
#' # Check results
#' print(off_mat[1:5, 1:5])
#' @importFrom pbapply pblapply
#' @importFrom hexbin hexbin
#' @importFrom sparseMatrixStats rowSums2
#' @importFrom stats xtabs
#' @importFrom flexmix FLXMRglm flexmix parameters clusters
#' @importFrom dbscan frNN
#' @export
local_offset_distance_with_background <- function(mat,
                                                  tx,
                                                  coords,
                                                  tx_x = "x",
                                                  tx_y = "y",
                                                  feature_label = "gene",
                                                  distance = 50,
                                                  nbins = 200,
                                                  cl = 1,
                                                  verbose = FALSE) {
  # store the gene names first in case a gene gets wiped out because of QV
  gene_names_tx <- unique(tx[[feature_label]])

  # filter by qv20 if the column exists (ie Xenium)
  if('qv' %in% colnames(tx)){
    message('QV column found! Filtering qv for high quality transcripts...')
    tx <- tx[tx[['qv']] >= 20,]
  }else{
    message('QV column not found! Skipping filtering...')
  }

  if(verbose){
    message('Calculating global background...')
  }

  # Create hexagonal bins
  hex_bins <- hexbin(tx[[tx_x]], tx[[tx_y]], xbins = nbins, IDs = TRUE) # Adjust xbins for resolution

  x_range <- diff(range(tx[,tx_x]))
  hex_radius <- x_range / hex_bins@xbins / sqrt(3)

  # Calculate the area of each hexbin
  hex_area <- (3 * sqrt(3) / 2) * hex_radius^2

  # Assign each transcript to a hexbin using the `hexbin` object
  tx$hexbin_id <- hex_bins@cID  # Use the `cID` slot to get the cell IDs for each point
  tx$feature_name <- tx[,feature_label]

  gene_bin_matrix <- xtabs(~ feature_name + hexbin_id, data = tx)

  if(nrow(gene_bin_matrix) < length(gene_names_tx)){
    message("Warning: Some genes in the original transcript data were not included in the hexbin matrix. This may be due to low quality transcripts being filtered out. Consider adjusting the QV threshold or checking the input data.")
    # pad the matrix with a row of 0s
    missing_genes <- setdiff(gene_names_tx, rownames(gene_bin_matrix))
    if(length(missing_genes) > 0){
      padding <- matrix(0, nrow = length(missing_genes), ncol = ncol(gene_bin_matrix))
      rownames(padding) <- missing_genes
      colnames(padding) <- colnames(gene_bin_matrix)
      gene_bin_matrix <- rbind(gene_bin_matrix, padding)
    }
  }

  # only keep genes that are present in the count matrix and the same order
  idx <- match(rownames(mat), rownames(gene_bin_matrix))
  idx <- idx[!is.na(idx)]

  gene_bin_matrix <- gene_bin_matrix[idx, , drop = FALSE]
  gene_bin_matrix[is.na(gene_bin_matrix)] <- 0

  bin_total <- colSums(gene_bin_matrix)

  # Extract empty bins inferred from GMM
  # Fit a Gaussian Mixture Model to colSums(gene_bin_matrix)
  if(verbose){
    message("Running GMM...")
  }
  mo1 <- FLXMRglm(family = "gaussian")
  mo2 <- FLXMRglm(family = "gaussian")
  bg_offset <- tryCatch(
        { flexfit <- flexmix(x ~ 1, data = data.frame(x=bin_total), k = 2, model = list(mo1, mo2))
          # Get the parameters of the GMM
          c1 <- parameters(flexfit, component=1)[[1]]
          c2 <- parameters(flexfit, component=2)[[1]]
          # Print the summary of the GMM

          # Identify the component with the smaller mean
          gmm_means <- c(c1[1], c2[1])
          smaller_mean_component <- which.min(gmm_means)

          empty_bin_matrix <- gene_bin_matrix[,clusters(flexfit) == smaller_mean_component]
          empty_bin_matrix <- empty_bin_matrix[,colSums(empty_bin_matrix) > 0]

          per_unit_sum <- rowSums(empty_bin_matrix)/(ncol(empty_bin_matrix) * hex_area)
          scaled_sum <- per_unit_sum * distance^2 * pi

          bg_offset <- ifelse(scaled_sum == 0, 1, ceiling(scaled_sum))
          bg_offset
        }, error = function(e){
          message("flexmix failed during GMM fit: ", e$message)
          message("Setting global background contamination to 1...")
          bg_offset <- rep(1, nrow(gene_bin_matrix))
          bg_offset
        }
  )

  bg_offset <- as.numeric(bg_offset)
  # for each obs, get neighbours within distance
  # and then get the total count
  get_neighbors_within_distance <- function(coords, distance) {
    coords_mat <- as.matrix(coords)
    neighbors <- vector("list", nrow(coords))
    neighbors <- pblapply(seq_len(nrow(coords)), function(i) {
      diff <- sweep(coords_mat, 2, coords_mat[i, ], FUN = "-")
      dists <- sqrt(rowSums2(diff^2))
      which(dists <= distance)
    }, cl = cl)

    #nn = frNN(coords_mat,eps=distance)

    #neighbors <- nn$id
    return(neighbors)
  }

  if(verbose){
    message("Finding neighbours...")
  }

  neighbors <- get_neighbors_within_distance(coords[, c(1,2)], distance)

  get_local_offset <- function(idx, neighbors, mat) {
    if (length(neighbors[[idx]]) == 0) {
      offset <- rep(0, nrow(mat)) + mat[, idx]
    } else {
      if (length(neighbors[[idx]]) == 1) {
        offset <- mat[, neighbors[[idx]]] + mat[, idx]
      } else {
        offset <- rowSums2(mat[, neighbors[[idx]]]) + mat[, idx]
      }
    }
    return(offset)
  }

  if(verbose){
    message("Calculating local offset...")
  }
  res <- pblapply(seq_len(ncol(mat)), get_local_offset, neighbors, mat, cl = cl)
  res_mat <- do.call(cbind, res)
  colnames(res_mat) <- colnames(mat)

  # add bg_offset to every column of res_mat
  res_mat <- sweep(res_mat, 1, bg_offset, "+")

  return(res_mat)
}

#' @title Neighbourhood offset fast
#' @description This function calculates the local neighbourhood offset together with ambient background for each cell in a count matrix.
#' This is a faster version of the local_offset_distance_with_background function that uses dbscan for neighbor finding.
#' Code credit to Sam Zimmerman with minor modifications.
#' @param mat A count matrix with genes as rows and cells as columns.
#' @param tx A transcript dataframe with x, y coordinates and qv values.
#' @param coords A dataframe with x, y coordinates of each cell as separate columns.
#' @param tx_x Column name for the x coordinates in the transcripts dataframe.
#' @param tx_y Column name for the y coordinates in the transcripts dataframe.
#' @param feature_label Column name for the gene of each transcript in the transcripts dataframe.
#' @param distance The maximum distance to consider for local background estimation.
#' @param nbins The number of bins to use for hexagonal binning, used for calculating background transcript contamination.
#' @param cl The number of cores to use for parallel processing.
#' @param verbose Logical, if TRUE, print progress messages.
#' @return A matrix of local background counts for each gene in each cell.
#' @details
#' The function calculates the offset used for each cell based on their local
#' neighbourhoods.In most cases you do not need to use this as denoist already
#' runs this internally but it is good for debugging if needed. Usage is the same as the non-fast version, will replace the original function in denoist in the future if this one is stable and faster.
#' @examples
#' # Load example data
#' set.seed(42)
#' mat <- matrix(rpois(1000, lambda = 10), nrow = 10, ncol = 100)
#' rownames(mat) <- paste0("gene", 1:10)
#' coords <- data.frame(x = rnorm(100), y = rnorm(100))
#' tx <- data.frame(x = c(rnorm(500), rnorm(500, 3)),
#'                  y = c(rnorm(500), rnorm(500, 3)),
#'                  qv = rep(30, 1000), gene = paste0('gene', 1:10))
#' # Run DenoIST
#' off_mat <- local_offset_distance_with_background_fast(mat, tx, coords,
#'                                                  distance = 1, nbins = 50,
#'                                                  cl = 1, verbose = TRUE)
#' # Check results
#' print(off_mat[1:5, 1:5])
#' @importFrom pbapply pblapply
#' @importFrom hexbin hexbin
#' @importFrom sparseMatrixStats rowSums2
#' @importFrom stats xtabs
#' @importFrom flexmix FLXMRglm flexmix parameters clusters
#' @importFrom dbscan frNN
#' @importFrom Matrix sparseMatrix
#' @importFrom methods as
#' @export
local_offset_distance_with_background_fast <- function(mat,
                                                        tx,
                                                        coords,
                                                        tx_x = "x",
                                                        tx_y = "y",
                                                        feature_label = "gene",
                                                        distance = 50,
                                                        nbins = 200,
                                                        cl = 1,
                                                        verbose = FALSE) {
  # store the gene names first in case a gene gets wiped out because of QV
  gene_names_tx <- unique(tx[[feature_label]])

  # filter by qv20 if the column exists (ie Xenium)
  if('qv' %in% colnames(tx)){
    message('QV column found! Filtering qv for high quality transcripts...')
    tx <- tx[tx[['qv']] >= 20,]
  }else{
    message('QV column not found! Skipping filtering...')
  }

  if(verbose){
    message('Calculating global background...')
  }

  # Create hexagonal bins
  hex_bins <- hexbin(tx[[tx_x]], tx[[tx_y]], xbins = nbins, IDs = TRUE) # Adjust xbins for resolution

  x_range <- diff(range(tx[[tx_x]]))
  hex_radius <- x_range / hex_bins@xbins / sqrt(3)

  # Calculate the area of each hexbin
  hex_area <- (3 * sqrt(3) / 2) * hex_radius^2

  # Assign each transcript to a hexbin using the `hexbin` object
  tx$hexbin_id <- hex_bins@cID  # Use the `cID` slot to get the cell IDs for each point
  tx$feature_name <- tx[[feature_label]]

  gene_bin_matrix <- xtabs(~ feature_name + hexbin_id, data = tx)

  if(nrow(gene_bin_matrix) < length(gene_names_tx)){
    message("Warning: Some genes in the original transcript data were not included in the hexbin matrix. This may be due to low quality transcripts being filtered out. Consider adjusting the QV threshold or checking the input data.")
    # pad the matrix with a row of 0s
    missing_genes <- setdiff(gene_names_tx, rownames(gene_bin_matrix))
    if(length(missing_genes) > 0){
      padding <- matrix(0, nrow = length(missing_genes), ncol = ncol(gene_bin_matrix))
      rownames(padding) <- missing_genes
      colnames(padding) <- colnames(gene_bin_matrix)
      gene_bin_matrix <- rbind(gene_bin_matrix, padding)
    }
  }

  # only keep genes that are present in the count matrix and the same order
  idx <- match(rownames(mat), rownames(gene_bin_matrix))
  idx <- idx[!is.na(idx)]

  gene_bin_matrix <- gene_bin_matrix[idx, , drop = FALSE]
  gene_bin_matrix[is.na(gene_bin_matrix)] <- 0

  bin_total <- colSums(gene_bin_matrix)

  # Extract empty bins inferred from GMM
  # Fit a Gaussian Mixture Model to colSums(gene_bin_matrix)
  if(verbose){
    message("Running GMM...")
  }
  mo1 <- FLXMRglm(family = "gaussian")
  mo2 <- FLXMRglm(family = "gaussian")
  bg_offset <- tryCatch(
    { flexfit <- flexmix(x ~ 1, data = data.frame(x=bin_total), k = 2, model = list(mo1, mo2))
    # Get the parameters of the GMM
    c1 <- parameters(flexfit, component=1)[[1]]
    c2 <- parameters(flexfit, component=2)[[1]]
    # Print the summary of the GMM

    # Identify the component with the smaller mean
    gmm_means <- c(c1[1], c2[1])
    smaller_mean_component <- which.min(gmm_means)

    empty_bin_matrix <- gene_bin_matrix[,clusters(flexfit) == smaller_mean_component]
    empty_bin_matrix <- empty_bin_matrix[,colSums(empty_bin_matrix) > 0]

    per_unit_sum <- rowSums(empty_bin_matrix)/(ncol(empty_bin_matrix) * hex_area)
    scaled_sum <- per_unit_sum * distance^2 * pi

    bg_offset <- ifelse(scaled_sum == 0, 1, ceiling(scaled_sum))
    bg_offset
    }, error = function(e){
      message("flexmix failed during GMM fit: ", e$message)
      message("Setting global background contamination to 1...")
      bg_offset <- rep(1, nrow(gene_bin_matrix))
      bg_offset
    }
  )

  bg_offset <- as.numeric(bg_offset)
  # for each obs, get neighbours within distance
  # and then get the total count
  get_neighbors_within_distance <- function(coords, distance) {
    coords_mat <- as.matrix(coords)
    nn = frNN(coords_mat,eps=distance)

    neighbors <- nn$id
    return(neighbors)
  }

  if(verbose){
    message("Finding neighbours...")
  }
  # note, self matches are not returned
  neighbors <- get_neighbors_within_distance(coords[, c(1,2)], distance)

  if(verbose){
    message("Calculating local offset...")
  }

  # alternatively try and combine all this into a single matrix algebra step
  n_cells <- ncol(mat)
  # 1. Create vectors of the row (i) and column (j) coordinates for our adjacency matrix
  # 'j' is the target cell index, repeated for as many neighbors as it has
  j_indices <- rep(seq_len(n_cells), lengths(neighbors))
  i_indices <- unlist(neighbors)

  A <- sparseMatrix(i = i_indices,
                    j = j_indices,
                    x = 1,
                    dims = c(n_cells, n_cells))
  A <- as(A > 0, "dMatrix")
  diag(A) <- 1
  all_offsets_mat <- mat %*% A
  all_offsets_mat = as.matrix(all_offsets_mat)
  colnames(all_offsets_mat) = colnames(mat)

  # add bg_offset to every column of res_mat
  res_mat <- sweep(all_offsets_mat, 1, bg_offset, "+")

  # add the self counts back to the neighbor offset
  res_mat <- res_mat + mat
  return(as.matrix(res_mat))
}

test_that("Neighbour offsets can be calculated", {
  # Test the function with the provided data
  test_tx <- readRDS(test_path("testdata", "test_tx.rds"))
  test_mat <- readRDS(test_path("testdata", "test_mat.rds"))
  test_coords <- readRDS(test_path("testdata", "test_coords.rds"))

  off <- local_offset_distance_with_background(mat = test_mat,
                                              coords = test_coords,
                                              tx = test_tx,
                                              distance = 50, nbins = 200,
                                              cl = 1, verbose = TRUE)
  expect_equal(nrow(off), nrow(test_mat))
  expect_equal(ncol(off), ncol(test_mat))
})
test_that("Transcript data with no QV can be handled", {
  # Test the function with the provided data
  test_tx <- readRDS(test_path("testdata", "test_tx.rds"))
  test_mat <- readRDS(test_path("testdata", "test_mat.rds"))
  test_coords <- readRDS(test_path("testdata", "test_coords.rds"))
  test_tx$qv <- NULL
  off <- local_offset_distance_with_background(mat = test_mat,
                                               coords = test_coords,
                                               tx = test_tx,
                                               distance = 50, nbins = 200,
                                               cl = 1, verbose = TRUE)
  expect_equal(nrow(off), nrow(test_mat))
  expect_equal(ncol(off), ncol(test_mat))
})
test_that("Fast version is consistent with the original", {
  # Test the function with the provided data
  test_tx <- readRDS(test_path("testdata", "test_tx.rds"))
  test_mat <- readRDS(test_path("testdata", "test_mat.rds"))
  test_coords <- readRDS(test_path("testdata", "test_coords.rds"))

  off1 <- local_offset_distance_with_background(mat = test_mat,
                                               coords = test_coords,
                                               tx = test_tx,
                                               distance = 50, nbins = 200,
                                               cl = 1, verbose = TRUE)
  off2 <- local_offset_distance_with_background_fast(mat = test_mat,
                                                    coords = test_coords,
                                                    tx = test_tx,
                                                    distance = 50, nbins = 200,
                                                    cl = 1, verbose = TRUE)

  expect_equal(off1, off2)
})

## code to prepare test dataset goes here
library(SubcellularSpatialData)
library(SpatialExperiment)
library(SingleCellExperiment)

data(tx_small)
spe <- tx2spe(tx_small, bin = "cell")
spe <- spe[,sample(1:ncol(spe), 1000)]
saveRDS(spe, file = "test_spe.rds")

#tx_test <- tx_small[sample(1:nrow(tx_small), 10000), ]

mat <- counts(spe)
# remove rows with rownames starting with BLANK or NegControl
mat <- mat[!grepl("BLANK|NegControl", rownames(mat)), ]
coords <- spatialCoords(spe)

saveRDS(mat, file = "test_mat.rds")
saveRDS(coords, file = "test_coords.rds")
saveRDS(tx_small, file = "test_tx.rds")
# further downsample to increase testing speed

#usethis::use_data(spe, overwrite = TRUE)
#usethis::use_data(mat, overwrite = TRUE)
#usethis::use_data(coords, overwrite = TRUE)
#usethis::use_data(tx_small, overwrite = TRUE)

#' @title Poisson mixture model solver
#' @param x A vector of counts.
#' @param s A vector of offsets.
#' @param max_iter Maximum number of iterations for the EM algorithm.
#' @param tol Tolerance for convergence.
#' @param n_inits Number of initial values for the mixing proportion inference. If input is a vector, directly use the vector has init values.
#' @param posterior_cutoff Cutoff for posterior probabilities to assign memberships.
#' @param verbose Logical, if TRUE, print progress messages.
#' @return{
#' A list containing the following elements:
#' \item{memberships}{A vector indicating the membership of each observation (0 or 1).}
#' \item{posterior}{A vector of posterior probabilities for each observation.}
#' \item{lambda1}{Estimated parameter for the first component.}
#' \item{lambda2}{Estimated parameter for the second component.}
#' \item{pi}{Estimated mixing proportion.}
#' \item{log_lik}{Log-likelihood of the fitted model.}
#' }
#' @description This function implements a 2-component Poisson mixture model
#' using the EM algorithm. It estimates the parameters of the model and assigns
#' memberships to each observation based on the posterior probabilities.
#' @details The function takes a vector of counts and a vector of offsets as input.
#' It uses the EM algorithm to iteratively update the parameters of the model
#' until convergence is reached or the maximum number of iterations is exceeded.
#' The function also allows for multiple initialisations of the mixing proportion
#' to find the best solution.
#' @examples
#' x <- rpois(100, lambda = 5)
#' s <- runif(100, min = 0, max = 1)
#' result <- solve_poisson_mixture(x, s)
#' print(result)
#' @export
solve_poisson_mixture <- function(x, s,
                                  max_iter = 5000,
                                  tol = 1e-6,
                                  n_inits = 10,
                                  posterior_cutoff = 0.6,
                                  verbose = FALSE) {

  n <- length(x)

  if(length(n_inits) == 1) {
    pi_inits <- runif(n_inits, min = 0, max = 0.5)
  }else{
    pi_inits <- n_inits
  }

  # check that pi_inits are all >0 and <=1
  if(any(pi_inits <= 0) || any(pi_inits > 1)) {
    stop("All initial values for pi must be in the range (0, 1]")
  }

  # Store indices of non-zero s
  non_zero_indices <- which(s > 0)

  # Remove entries with s = 0
  x <- x[non_zero_indices]
  s <- s[non_zero_indices]

  best_result <- NULL
  best_log_lik <- -Inf

  for (pi_init in pi_inits) {
    # Initialize parameters
    lambda1 <- mean(x) / mean(s)
    lambda2 <- mean(x) / (2 * mean(s))
    pi <- pi_init

    if (verbose) {
      message("Initial parameters for pi =", pi_init, ":\n")
      message("lambda1:", lambda1, "lambda2:", lambda2, "pi:", pi, "\n")
    }

    log_likelihood <- function(x, s, lambda1, lambda2, pi) {
      sum(log(pi * dpois(x, s * lambda1) + (1 - pi) * dpois(x, s * lambda2)))
    }

    log_lik <- log_likelihood(x, s, lambda1, lambda2, pi)

    for (iter in seq_len(max_iter)) {
      # E-step: calculate responsibilities
      tau1 <- pi * dpois(x, s * lambda1)
      tau2 <- (1 - pi) * dpois(x, s * lambda2)
      gamma <- tau1 / (tau1 + tau2)

      # M-step: update parameters
      lambda1 <- sum(gamma * x) / sum(gamma * s)
      lambda2 <- sum((1 - gamma) * x) / sum((1 - gamma) * s)
      pi <- mean(gamma)

      if (verbose) {
        message("Iteration", iter, "parameters:\n")
        message("lambda1:", lambda1, "lambda2:", lambda2, "pi:", pi, "\n")
      }

      # Check for convergence
      new_log_lik <- log_likelihood(x, s, lambda1, lambda2, pi)
      if (!is.finite(new_log_lik) || abs(new_log_lik - log_lik) < tol) {
        if (verbose) {
          message("Converged after", iter, "iterations\n")
          message("Final log-likelihood:", log_lik, "\n")
        }
        break
      }

      log_lik <- new_log_lik
    }

    if (log_lik > best_log_lik) {
      best_log_lik <- log_lik
      if(abs(lambda1 - lambda2) > 1e-2) {
        # Store the best parameters
        best_result <- list(lambda1 = lambda1,
                            lambda2 = lambda2,
                            pi = pi,
                            log_lik = log_lik,
                            gamma = gamma)
      }else{
        # If model collapse occurs, keep everything
        best_result <- list(lambda1 = lambda1,
                            lambda2 = lambda2,
                            pi = pi,
                            log_lik = log_lik,
                            gamma = rep(1, length(x)))
      }
    }
  }

  # Assign memberships
  memberships <- ifelse(best_result$gamma >= posterior_cutoff, 1, 0)

  # TODO: if memberships are all 0, set to 1
  if (all(memberships == 0)) {
    memberships <- rep(1, length(memberships))
  }

  # Pad the results to match the original input length
  full_memberships <- rep(1, n)
  full_memberships[non_zero_indices] <- memberships

  full_posterior <- rep(1, n)
  full_posterior[non_zero_indices] <- best_result$gamma

  return(list(memberships = full_memberships,
              posterior = full_posterior,
              lambda1 = best_result$lambda1,
              lambda2 = best_result$lambda2,
              pi = best_result$pi,
              log_lik = best_result$log_lik))
}

# Test solve_poisson_mixture
test_that("solve_poisson_mixture works with simple input", {
  x <- c(3, 5, 2, 8, 6)
  s <- c(1, 1, 1, 1, 1)
  result <- solve_poisson_mixture(x, s, verbose = FALSE)

  expect_type(result, "list")
  expect_true(all(result$memberships %in% c(0, 1)))
  expect_equal(length(result$memberships), length(x))
  expect_equal(length(result$posterior), length(x))
  expect_true(result$lambda1 > 0)
  expect_true(result$lambda2 > 0)
  expect_true(result$pi >= 0 && result$pi <= 1)
})

test_that("solve_poisson_mixture handles zero s values", {
  x <- c(3, 5, 2, 8, 6)
  s <- c(1, 0, 1, 0, 1)
  result <- solve_poisson_mixture(x, s, verbose = FALSE)

  expect_equal(length(result$memberships), length(x))
  expect_equal(result$memberships[which(s == 0)], rep(1, sum(s == 0)))
})

test_that("solve_poisson_mixture handles single init value as input", {
  x <- c(3, 5, 2, 8, 6)
  s <- c(1, 0, 1, 0, 1)
  result <- solve_poisson_mixture(x, s, n_inits = 5, verbose = FALSE)

  expect_equal(length(result$memberships), length(x))
  expect_equal(result$memberships[which(s == 0)], rep(1, sum(s == 0)))
})

test_that("solve_poisson_mixture handles vector init values as input", {
  x <- c(3, 5, 2, 8, 6)
  s <- c(1, 0, 1, 0, 1)
  i <- c(0.1, 0.2, 0.3, 0.4, 0.5)
  result <- solve_poisson_mixture(x, s, n_inits = i, verbose = FALSE)

  expect_equal(length(result$memberships), length(x))
  expect_equal(result$memberships[which(s == 0)], rep(1, sum(s == 0)))
})

test_that("solve_poisson_mixture checks valid init values", {
  x <- c(3, 5, 2, 8, 6)
  s <- c(1, 0, 1, 0, 1)

  expect_error(solve_poisson_mixture(x, s, n_inits = c(-0.1, 0.2), verbose = FALSE))
  expect_error(solve_poisson_mixture(x, s, n_inits = c(0.1, 1.5), verbose = FALSE))
})

utils::globalVariables(c("feature_name", "hexbin_id", "count"))

#' @importFrom pbapply pblapply
#' @importFrom stats dpois runif
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom SummarizedExperiment assay
#' @importFrom parallel makeCluster clusterExport parLapply stopCluster
#' @export
#' @title DenoIST
#' @description
#' DenoIST (Denoising Image-based Spatial Transcriptomics) is a method for
#' identifying and removing contamination artefacts in
#' image-based single-cell transcriptomics (IST) data.
#' It uses a transposed Poisson mixture model to identify contamination.
#' @param mat A matrix of counts (genes x cells), or a SpatialExperiment object.
#' @param coords A data frame of coordinates (n_cells x 2). Only used if not using a SpatialExperiment object as input.
#' @param tx A data frame of transcript with x, y and qv columns.
#' @param tx_x Column name for the x coordinates in the transcripts dataframe. Default is 'x'.
#' @param tx_y Column name for the y coordinates in the transcripts dataframe. Default is 'y'.
#' @param feature_label Column name for the gene of each transcript in the transcripts dataframe. Default is 'gene'.
#' @param distance The maximum distance to consider for local background estimation.
#' @param nbins The number of bins to use for hexagonal binning.
#' @param posterior_cutoff The cutoff for posterior probability to determine contamination.
#' @param n_inits The number of initialisations for the mixing proportion in the Poisson mixture model. If input is a vector, directly use the vector of values as init values.
#' @param cl The number of cores to use for parallel processing.
#' @param out_dir The output directory to save the results.
#' @param neighbour_mode The method to use for finding neighbors. Options are 'fast' (default) and 'original'. 'fast' uses a faster implementation with dbscan, while 'original' uses the original implementation.
#' @param verbose Logical, if TRUE, print progress messages.
#' @return A list containing the following elements:
#' \item{memberships}{A matrix of memberships for each gene in each cell.}
#' \item{adjusted_counts}{A matrix of adjusted counts for each gene in each cell.}
#' \item{params}{A list of parameters for each gene.}
#' @details
#' The function calculates local background using hexagonal binning and applies
#' a Poisson mixture model to identify contamination.
#' It returns a matrix of memberships and adjusted counts for each gene in each cell.
#' @examples
#' # Load example data
#' set.seed(42)
#' mat <- matrix(rpois(1000, lambda = 10), nrow = 10, ncol = 100)
#' rownames(mat) <- paste0("gene", 1:10)
#' coords <- data.frame(x = rnorm(100), y = rnorm(100))
#' tx <- data.frame(x = c(rnorm(500), rnorm(500, 3)),
#'                  y = c(rnorm(500), rnorm(500, 3)),
#'                  qv = rep(30, 1000), gene = paste0('gene', 1:10))
#' # Run DenoIST
#' result <- denoist(mat, tx, coords, distance = 1, nbins = 50, cl = 1,
#'                   out_dir = NULL, verbose = TRUE)
#' # Check results
#' print(result$memberships[1:5, 1:5])
#' print(result$adjusted_counts[1:5, 1:5])
#' print(result$params[[1]])
denoist <- function(mat, tx, coords = NULL,
                    tx_x = "x",
                    tx_y = "y",
                    feature_label = "gene",
                    distance = 50,
                    nbins = 200,
                    posterior_cutoff = 0.6,
                    n_inits = 10,
                    cl = 1,
                    out_dir = NULL,
                    neighbour_mode = 'fast',
                    verbose = FALSE){

  # TODO:check input type
  if(inherits(mat, "SpatialExperiment")){
    coords <- spatialCoords(mat)
    mat <- assay(mat)
    #remove NegControl and BLANKS
    mat <- mat[!grepl("NegControl|BLANK|Unassigned", rownames(mat)),]
  }else if(is.null(coords)){
    stop("coords must be provided")
  }

  # calculate neighbour_offset
  if(verbose){
    message("Calculating neighbour offset...")
  }

  if(neighbour_mode == 'fast'){
    off_mat <- local_offset_distance_with_background_fast(mat = mat,
                                                        coords = coords,
                                                        tx = tx,
                                                        tx_x = tx_x,
                                                        tx_y = tx_y,
                                                        feature_label = feature_label,
                                                        distance = distance,
                                                        nbins = nbins,
                                                        cl = cl,
                                                        verbose = verbose)
  }else if(neighbour_mode == 'original'){
  off_mat <- local_offset_distance_with_background(mat = mat,
                                                   coords = coords,
                                                   tx = tx,
                                                   tx_x = tx_x,
                                                   tx_y = tx_y,
                                                   feature_label = feature_label,
                                                   distance = distance,
                                                   nbins = nbins,
                                                   cl = cl,
                                                   verbose = verbose)
  }else{
    stop("Invalid neighbour_mode. Choose 'fast' or 'original'.")
  }

  # Clear tx just in case to save mem
  rm(tx)
  gc()

  # Function to parallelize over
  apply_poisson_mixture_single <- function(mat, off_mat, posterior_cutoff, n_inits) {
    function(idx) { # friendship ended with OOP, FP is now my best friend
      c_vec <- mat[,idx]
      s_vec <- off_mat[,idx]
      result <- tryCatch({
        out <- solve_poisson_mixture(c_vec, s_vec, posterior_cutoff = posterior_cutoff, n_inits = n_inits)
        return(out)
      }, error = function(e) {
        out <- list(memberships = rep(1, length(c_vec)),
                    posterior = rep(1, length(c_vec)),
                    lambda1 = NA, lambda2 = NA, pi = NA)
        return(out)
      })
      return(result)
    }
  }

  if(cl > 1){
    if(verbose){
      message("Setting up parallel processing...")
    }
    clust <- makeCluster(cl-1, type = "FORK")

    if(verbose){
      message("Exporting variables to workers...")
    }
    clusterExport(clust, c("mat", "off_mat", "solve_poisson_mixture", "posterior_cutoff", "n_inits"), envir = environment())

    # Apply the Poisson mixture model
    if(verbose){
      message("Applying the Poisson mixture model...")
    }
    results <- parLapply(cl = clust, seq_len(ncol(mat)), apply_poisson_mixture_single(mat, off_mat, posterior_cutoff, n_inits))
    stopCluster(clust)
  }else{
    # use pblapply normally without multiple processing
    if(verbose){
      message("Applying the Poisson mixture model without parallel processing...")
    }
    results <- pblapply(seq_len(ncol(mat)), apply_poisson_mixture_single(mat, off_mat, posterior_cutoff, n_inits), cl = cl)
  }


  # return neighbour_offset, adjusted_counts, posterior, params
  if(verbose){
    message("Tidying up results...")
  }
  memberships_matrix <- do.call(cbind, lapply(results, function(res) res["memberships"]))
  memberships_matrix <- do.call(cbind, memberships_matrix)
  colnames(memberships_matrix) <- colnames(mat)
  rownames(memberships_matrix) <- rownames(mat)

  adjusted_counts <- mat * memberships_matrix
  colnames(adjusted_counts) <- colnames(mat)
  rownames(adjusted_counts) <- rownames(mat)

  # save the results
  if(!is.null(out_dir)){
    if(!dir.exists(out_dir)){
      dir.create(out_dir, recursive = TRUE)
    }
    saveRDS(list(memberships = memberships_matrix,
                 adjusted_counts = adjusted_counts,
                 params = results),
            file = paste0(out_dir, "/denoist_results.rds"))
  }

  return(list(memberships = memberships_matrix,
              adjusted_counts = adjusted_counts,
              params = results))
}

test_mat <- readRDS(test_path("testdata", "test_mat.rds"))
test_coords <- readRDS(test_path("testdata", "test_coords.rds"))
test_tx <- readRDS(test_path("testdata", "test_tx.rds"))
test_spe <- readRDS(test_path("testdata", "test_spe.rds"))

test_that("DenoIST works with SpatialExperiment input", {
   # Test the function with the provided data
  res <- denoist(mat = test_spe,
              coords = NULL,
              tx = test_tx,
              distance = 50, nbins = 200, cl = 1,
              out_dir = "denoist_results", verbose = TRUE)
  expect_length(res, 3)
})


test_that("DenoIST works with Matrix input", {
  # Test the function with the provided data
  res <- denoist(mat = test_mat,
               coords = test_coords,
               tx = test_tx,
               distance = 50, nbins = 200, cl = 1, verbose = TRUE)
  expect_length(res, 3)
})

test_that("DenoIST can be sped up with less inits", {
  # Test the function with the provided data
  res <- denoist(mat = test_mat,
                 coords = test_coords,
                 tx = test_tx,
                 distance = 50, nbins = 200, cl = 1, n_inits = 5, verbose = TRUE)
  expect_length(res, 3)
})

test_that("DenoIST takes vector as init input", {
  i <- c(0.1, 0.2, 0.3, 0.4, 0.5)
  # Test the function with the provided data
  res <- denoist(mat = test_mat,
                 coords = test_coords,
                 tx = test_tx,
                 distance = 50, nbins = 200, cl = 1, n_inits = i, verbose = TRUE)
  expect_length(res, 3)
})

---
title: "Denoising Imaged-based Spatial Transcriptomics data with DenoIST"
output: BiocStyle::html_document
author: "Aaron Kwok"
package: DenoIST
vignette: >
  %\VignetteIndexEntry{denoist_spe}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

DenoIST (Denoising Image-based Spatial Transcriptomics) is a method for identifying and removing contamination artefacts in image-based single-cell transcriptomics (IST) data. This vignette shows how to use it with a `SpatialExperiment` object or a matrix with coordinates as a separate input.

Why does IST data need denoising? As cell segmentation is imperfect, sometimes segmented cells present gene expression profiles that are contradictory and highly improbable. DenoIST can identify these cases and remove likely `contaminating' gene transcripts. If you are interested in more details, please refer to the [preprint](https://www.biorxiv.org/content/10.1101/2025.11.13.688387v1).

# Installation

You can install `DenoIST` via Bioconductor:

```
if(!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("DenoIST")
```

# Load data

For demonstration, we will use a small Xenium sample from a lung fibrosis study (Vannan & Lyu et. al, 2025). It can be downloaded at [GSE250346](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE250346). We downsampled this dataset to 300 cells for demonstration purpose.

For this vignette, we will use a pre-made `SpatialExperiment` object.

```{r setup}
suppressPackageStartupMessages({
  library(DenoIST)
  library(SpatialExperiment)
  library(ggplot2)
  library(patchwork)
})
```

```{r load_spe}
spe <- readRDS(system.file("extdata", "example_spe.rds", package = "DenoIST"))
spe
```

Since the transcript file is too big to upload, for demonstration we will read in a very small subset to show the what the format is like:

```{r load_data}
tx <- readRDS(system.file("extdata", "tx_sub.rds", package = "DenoIST"))
tx
```
These 2 files would be the input for denoising.

# Denoising the data

You should only need to use 1 function most of the time, unless you are trying to debug or understand the process. The main function is `denoist()`, which takes a `SpatialExperiment` object (or a matrix with coordinates), plus the transcript data frame as input. It will return a list containing the memberships, adjusted counts, and parameters for each gene.

The `distance` parameter specifies the maximum distance to consider for local background estimation. The `nbins` parameter specifies the number of bins to use for hexagonal binning, which is used for calculating background transcript contamination. They have default values but you can adjust them based on your data. For example if your data is very small in size then perhaps a lower `distance` and `nbins` would be better.

You should also check whether transcript data frame has the correct columns. The `tx_x` and `tx_y` parameters specify the column names for the x and y coordinates, respectively. The `feature_label` parameter specifies the column name for the gene of each transcript. In this example, they are called `x_location`, `y_location` and `feature_name`. You can also speed up the process with more cpus via the `cl` option (which is highly recommended).

Lastly, you can specify an output directory with `out_dir` to save the results automatically. If you don't want to, just leave it empty.

```{r denoist}
# DenoIST
result <- denoist(mat = spe, tx = tx, coords = NULL, tx_x = "x_location", tx_y = "y_location", feature_label = "feature_name", distance = 50, nbins = 200, cl = 1)
```

Not that this is exactly the same as extracting the matrix and coordinates out manually: 

```
count_mat <- assay(spe)
coords <- spatialCoords(spe)

result <- denoist(mat = count_mat, tx = tx, coords = coords, tx_x = "x_location", tx_y = "y_location", feature_label = "feature_name", distance = 50, nbins = 200, cl = 1)
```

This is useful if you want to run DenoIST on a matrix that is not a `SpatialExperiment` object.

# Check results

The most useful output in `result` should be the `adjusted_counts`. Here we read in a pre-made result with more cells to better demonstrate the intended effect.

```{r read_results}
result <- readRDS(system.file("extdata", "result.rds", package = "DenoIST"))
```


```{r check_results}
result$adjusted_counts[1:5, 1:5]
```
The `memberships` and `params` are useful for debugging if the adjusted counts are not what you expect.

```{r}
result$memberships[1:5, 1:5]
```
```{r}
result$params[[42]]
```

We can also quickly visualise the difference.

```{r}
# a custom helper function for plotting
plot_feature_scatter <- function(coords, mat, feature, size = 0.3, output_filename = NULL) {
  # Create a data frame from the coordinates and features
  plot_data <- data.frame(x = coords[, 1], y = coords[, 2], feature = mat[feature,])
  
  # Create the scatterplot
  p <- ggplot(plot_data, aes(x = x, y = y, color = feature)) +
    geom_point(size = size, alpha = 0.5) +  # Make the dots smaller
    theme_minimal() +
    labs(title = feature, x = "X Coordinate", y = "Y Coordinate", color = "Feature") +
    theme(legend.position = "right") +
    scale_color_viridis_c()  # Use a color palette with higher contrast
  
  # Save the plot if an output filename is provided
  if (!is.null(output_filename)) {
    ggsave(output_filename, p, width = 10, height = 8, units = "in", dpi = 300)
  }
  
  # Return the plot
  return(p)
}
```

```{r,fig.height=8, fig.width=10}
orig <- plot_feature_scatter(coords = spatialCoords(spe), log(assay(spe)+1), feature = "EPAS1") + ggtitle('Original')
adj <- plot_feature_scatter(coords = spatialCoords(spe), log(result$adjusted_counts+1), feature = "EPAS1") + ggtitle('DenoIST adjusted')
(orig + adj) + plot_layout(guides = "collect")
```

# Session information {.unnumbered}

```{r sessionInfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neighbour_offset.R
\name{local_offset_distance_with_background}
\alias{local_offset_distance_with_background}
\title{Neighbourhood offset}
\usage{
local_offset_distance_with_background(
  mat,
  tx,
  coords,
  tx_x = "x",
  tx_y = "y",
  feature_label = "gene",
  distance = 50,
  nbins = 200,
  cl = 1,
  verbose = FALSE
)
}
\arguments{
\item{mat}{A count matrix with genes as rows and cells as columns.}

\item{tx}{A transcript dataframe with x, y coordinates and qv values.}

\item{coords}{A dataframe with x, y coordinates of each cell as separate columns.}

\item{tx_x}{Column name for the x coordinates in the transcripts dataframe.}

\item{tx_y}{Column name for the y coordinates in the transcripts dataframe.}

\item{feature_label}{Column name for the gene of each transcript in the transcripts dataframe.}

\item{distance}{The maximum distance to consider for local background estimation.}

\item{nbins}{The number of bins to use for hexagonal binning, used for calculating background transcript contamination.}

\item{cl}{The number of cores to use for parallel processing.}

\item{verbose}{Logical, if TRUE, print progress messages.}
}
\value{
A matrix of local background counts for each gene in each cell.
}
\description{
This function calculates the local neighbourhood offset together with ambient background for each cell in a count matrix.
}
\details{
The function calculates the offset used for each cell based on their local
neighbourhoods.In most cases you do not need to use this as denoist already
runs this internally but it is good for debugging if needed.
}
\examples{
# Load example data
set.seed(42)
mat <- matrix(rpois(1000, lambda = 10), nrow = 10, ncol = 100)
rownames(mat) <- paste0("gene", 1:10)
coords <- data.frame(x = rnorm(100), y = rnorm(100))
tx <- data.frame(x = c(rnorm(500), rnorm(500, 3)),
                 y = c(rnorm(500), rnorm(500, 3)),
                 qv = rep(30, 1000), gene = paste0('gene', 1:10))
# Run DenoIST
off_mat <- local_offset_distance_with_background(mat, tx, coords,
                                                 distance = 1, nbins = 50,
                                                 cl = 1, verbose = TRUE)
# Check results
print(off_mat[1:5, 1:5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/denoist.R
\name{denoist}
\alias{denoist}
\title{DenoIST}
\usage{
denoist(
  mat,
  tx,
  coords = NULL,
  tx_x = "x",
  tx_y = "y",
  feature_label = "gene",
  distance = 50,
  nbins = 200,
  posterior_cutoff = 0.6,
  n_inits = 10,
  cl = 1,
  out_dir = NULL,
  neighbour_mode = "fast",
  verbose = FALSE
)
}
\arguments{
\item{mat}{A matrix of counts (genes x cells), or a SpatialExperiment object.}

\item{tx}{A data frame of transcript with x, y and qv columns.}

\item{coords}{A data frame of coordinates (n_cells x 2). Only used if not using a SpatialExperiment object as input.}

\item{tx_x}{Column name for the x coordinates in the transcripts dataframe. Default is 'x'.}

\item{tx_y}{Column name for the y coordinates in the transcripts dataframe. Default is 'y'.}

\item{feature_label}{Column name for the gene of each transcript in the transcripts dataframe. Default is 'gene'.}

\item{distance}{The maximum distance to consider for local background estimation.}

\item{nbins}{The number of bins to use for hexagonal binning.}

\item{posterior_cutoff}{The cutoff for posterior probability to determine contamination.}

\item{n_inits}{The number of initialisations for the mixing proportion in the Poisson mixture model. If input is a vector, directly use the vector of values as init values.}

\item{cl}{The number of cores to use for parallel processing.}

\item{out_dir}{The output directory to save the results.}

\item{neighbour_mode}{The method to use for finding neighbors. Options are 'fast' (default) and 'original'. 'fast' uses a faster implementation with dbscan, while 'original' uses the original implementation.}

\item{verbose}{Logical, if TRUE, print progress messages.}
}
\value{
A list containing the following elements:
\item{memberships}{A matrix of memberships for each gene in each cell.}
\item{adjusted_counts}{A matrix of adjusted counts for each gene in each cell.}
\item{params}{A list of parameters for each gene.}
}
\description{
DenoIST (Denoising Image-based Spatial Transcriptomics) is a method for
identifying and removing contamination artefacts in
image-based single-cell transcriptomics (IST) data.
It uses a transposed Poisson mixture model to identify contamination.
}
\details{
The function calculates local background using hexagonal binning and applies
a Poisson mixture model to identify contamination.
It returns a matrix of memberships and adjusted counts for each gene in each cell.
}
\examples{
# Load example data
set.seed(42)
mat <- matrix(rpois(1000, lambda = 10), nrow = 10, ncol = 100)
rownames(mat) <- paste0("gene", 1:10)
coords <- data.frame(x = rnorm(100), y = rnorm(100))
tx <- data.frame(x = c(rnorm(500), rnorm(500, 3)),
                 y = c(rnorm(500), rnorm(500, 3)),
                 qv = rep(30, 1000), gene = paste0('gene', 1:10))
# Run DenoIST
result <- denoist(mat, tx, coords, distance = 1, nbins = 50, cl = 1,
                  out_dir = NULL, verbose = TRUE)
# Check results
print(result$memberships[1:5, 1:5])
print(result$adjusted_counts[1:5, 1:5])
print(result$params[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmm_model.R
\name{solve_poisson_mixture}
\alias{solve_poisson_mixture}
\title{Poisson mixture model solver}
\usage{
solve_poisson_mixture(
  x,
  s,
  max_iter = 5000,
  tol = 1e-06,
  n_inits = 10,
  posterior_cutoff = 0.6,
  verbose = FALSE
)
}
\arguments{
\item{x}{A vector of counts.}

\item{s}{A vector of offsets.}

\item{max_iter}{Maximum number of iterations for the EM algorithm.}

\item{tol}{Tolerance for convergence.}

\item{n_inits}{Number of initial values for the mixing proportion inference. If input is a vector, directly use the vector has init values.}

\item{posterior_cutoff}{Cutoff for posterior probabilities to assign memberships.}

\item{verbose}{Logical, if TRUE, print progress messages.}
}
\value{
{
A list containing the following elements:
\item{memberships}{A vector indicating the membership of each observation (0 or 1).}
\item{posterior}{A vector of posterior probabilities for each observation.}
\item{lambda1}{Estimated parameter for the first component.}
\item{lambda2}{Estimated parameter for the second component.}
\item{pi}{Estimated mixing proportion.}
\item{log_lik}{Log-likelihood of the fitted model.}
}
}
\description{
This function implements a 2-component Poisson mixture model
using the EM algorithm. It estimates the parameters of the model and assigns
memberships to each observation based on the posterior probabilities.
}
\details{
The function takes a vector of counts and a vector of offsets as input.
It uses the EM algorithm to iteratively update the parameters of the model
until convergence is reached or the maximum number of iterations is exceeded.
The function also allows for multiple initialisations of the mixing proportion
to find the best solution.
}
\examples{
x <- rpois(100, lambda = 5)
s <- runif(100, min = 0, max = 1)
result <- solve_poisson_mixture(x, s)
print(result)
}
